/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Contributor(s): Mark Bokil <mbokil@bokil.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * author: Mark Bokil
 * version: 0.1
 * date: 9/10/03
 *
 * ***** END LICENSE BLOCK ***** */



// initialize app main global references
var content;
var contentCopy;
var browser;
var console;
var inputFile;
var mainWindow;
var status;
var bundle;
var autoSave = false;
var documentHistory;
var historyMax = 25;
var bufferList;
var buffer;
var historyItems = 0;
var isModified = false;
var isBackup = true;
var isSaveNagEnabled = true;

// default UI prefs if bundle is missing
var fontFace = "Arial";
var fontSize = ".8em";
var fontColor = "black";
var fontWeight = "regualr";
var backgroundColor = "white";

function tellerror(msg, url, linenumber){
    dump('Javascript Error message= '+msg+'\nURL= '+url+'\nLine Number= '+linenumber+"\n")
    return true
}

function init() {
    // turn on javascript error dumping
    dump("\nStarting up Mozedit App...\n");

    //include external libraries in JSLIB
    include("chrome://jslib/content/io/file.js");
	include("chrome://jslib/content/io/fileUtils.js");
    include("chrome://jslib/content/io/dirUtils.js");
    include("chrome://jslib/content/io/dir.js");

    //mozedit include library files
    include("chrome://mozedit/content/scripts/buffer.js");
    include("chrome://mozedit/content/scripts/io.js");
    include("chrome://mozedit/content/scripts/editingfunctions.js");
    include("chrome://mozedit/content/scripts/clipboard.js");
    include("chrome://mozedit/content/scripts/utility.js");

	//include("chrome://global/content/strres.js");

    // mainWindow is a reference to the main app window
    mainWindow = document.getElementById('main-window');

    // user input field
    inputFile = document.getElementById('file-input');
	inputFile.focus();

    // status bar
    status = document.getElementById('status');

    // content is the main text editing area
    content = document.getElementById('content-editor');

    // line number pane
    lines = document.getElementById('line-numbers');

    // browser is rendering area in app
    browser = document.getElementById('browser-display');

	// browser is rendering area in app
    console = document.getElementById('javascript-console')

    // center window on
    centerWindowOnScreen();

    // document history object
	documentHistory = new Array();

	// restore properties
    doProperties();
    getBundle();
    getHistory();
    buildGoMenu();
    restoreUIPrefs();

    // set up buffers
	buffer = new Buffer(content.value, inputFile.value);
    bufferList = new BufferList();
	bufferList.addBuffer( buffer );
}




// onunload handler checks file saved state
function handleOnUnload() {
//     var msg = "\nYour file has not been saved yet. Save it now?";
// 
//     if ( isModified ) {
// 	    if (confirm(msg)) {
// 		    saveFilePicker();
// 	    }
// 	}
}


// setup application property files
function doProperties() {
    var du = new DirUtils();
    var pathChrome = "/chrome";
	var pathMozedit = "/chrome/mozeditproperties";
    var propertiesFile = "/mozedit.properties";
    var historyFile = "/history.properties";

	try {
         var homePath = du.getMozUserHomeDir();
         var d = new Dir(homePath + "/chrome/mozeditproperties");

         if ( !d.exists() ) {
             d.create();
             dump("new mozeditproperties directory in doProperties created\n");
         }
         if ( !d.exists() ) {
             dump("could not write mozilla directory in doProperties\n");
             return false;
         }

         //write propery files
         var fProperties = new File(homePath + pathMozedit + propertiesFile);
         var fHistory = new File(homePath + pathMozedit + historyFile);

         fProperties.create();
         fHistory.create();

         if ( !fProperties.exists() && !fHistory.exists() ) {
             dump("could not write property files in doProperties\n")
             return false;
         }
    } catch(e) {
        dump(e+ " error occured in doProperties\n");
        return false;
    }
    return true;
}


// sets isModified to true when file changes
function setModifiedState(state) {
    if (state || state == null) {
        isModified = true;
        dump("content changed\n");
	} else {
	    isModified = false;
	}
}


// dump send output to STDOUT terminal
function doDump() {
    const PREFS_CID    = "@mozilla.org/preferences;1";
	const PREFS_I_PREF = "nsIPref";
	const PREF_STRING  = "browser.dom.window.dump.enabled";

	try {
	    var Pref = new Components.Constructor(PREFS_CID, PREFS_I_PREF);
		var pref = new Pref();
		pref.SetBoolPref(PREF_STRING, true);
	} catch(e) {}
}




function addMenuBufferItem() {;
    var menuList = document.getElementById('menu-buffer-list');
    var menuPopup = document.getElementById('menu-buffer-popup');
    var menuItem = document.createElement('menuitem');
	var x = BufferList.list.length;
    menuPopup.appendChild(menuItem);
    menuItem.setAttribute('label','Buffer '+x);
	menuItem.setAttribute('oncommand','doBuff('+x+')');
	menuItem.setAttribute('id','buffer-item'+x);
	menuList.selectedIndex = x;
	x++;
    tempBuffer = new Buffer("","");
	buffer.saveBuffer(content.value, inputFile.value);
	bufferList.addBuffer( tempBuffer );
	buffer = tempBuffer;
	content.value = "";
	inputFile.value = "";
	updateStatus("Buffer created..");
}

function removeMenuBufferItem() {
    var x = BufferList.list.length;
    if ( x > 1) {
        var menuList = document.getElementById('menu-buffer-list');
        var menuPopup = document.getElementById('menu-buffer-popup');
	    var currentBuffer = document.getElementById( 'buffer-item'+(x-1) );
        throwaway_node = menuPopup.removeChild( currentBuffer );
		menuList.selectedIndex = x-2;
		dump("removed node: "+throwaway_node);
	    x--;
	}
	bufferList.removeBuffer();
    buffer = bufferList.getBuffer(BufferList.list.length-1);
	content.value = buffer.getContent();
	inputFile.value = buffer.getInput();
}


function doBuff(id) {
    dump(BufferList.currentBuffer);
	buffer.saveBuffer(content.value, inputFile.value);
	buffer = bufferList.getBuffer(id);
    content.value = buffer.getContent();
	inputFile.value = buffer.getInput();
}


// restore UI preferences
function restoreUIPrefs() {
   content.setAttribute('style','font-size:'+
	    fontSize+'pt;font-family:'+fontFace+';color:'+fontColor+';background:orange');
   inputFile.setAttribute('style','font-size:'+
	    fontSize+'pt;font-family:'+fontFace+';color:'+fontColor);
   status.setAttribute('style','font-size:'+
	    fontSize+'pt;font-family:'+fontFace+';color:'+fontColor);
}


// get properties value from keyword
function getProperty(bundle,keyword) {
    var properties = bundle.split("\n");

    for (x=0; x < properties.length; x++) {
        if ( properties[x].indexOf(keyword) != -1 ) {
		    var position = properties[x].indexOf("=");
			var value = properties[x].substring( position+1, properties[x].length );
			return value;
		}
	}

}


// save properties file
 function saveProps() {
     var propStr = "";
         propStr = propStr + "font.face=" + fontFace + "\n";
         propStr = propStr + "font.size=" + fontSize + "\n";
         propStr = propStr + "font.color=" + fontColor + "\n";
 		 propStr = propStr + "background.color=" + backgroundColor + "\n";
     if (navigator.platform == "Win32") {
         propStr = propStr.replace(/\//g,"\\");
     }
     try {
         var du = new DirUtils();
         var unixPath = "/chrome/mozeditproperties/mozedit.properties";
         var winPath = "\\chrome\\mozeditproperties\\mozedit.properties";
         var path = unixPath;

         if (navigator.platform == "Win32") {
             path = winPath;
         }

         path = du.getMozUserHomeDir() + path;
         var f = new File(path);

         if (f.exists()) {
             f.open('w');
             f.write(propStr);
             f.close();
             updateStatus("Properties saved...");
         } else {
             f.create();
             if (f.exists()) {
             f.open('w');
             f.write(propStr);
             updateStatus("File saved...");
             } else {
             dump("props could not be saved");
             return false;
             }
             f.close();
         }
    } catch(e) {
        dump(e + " error occured in saveProps()");
        return false;
    }
    return true;
}



//loads in properties bundle
function getBundle() {
    var du = new DirUtils();
    var unixPath = "/chrome/mozeditproperties/mozedit.properties";
    var winPath = "\\chrome\\mozeditproperties\\mozedit.properties";
    var path = unixPath;

    if (navigator.platform == "Win32") {
	    path = winPath;
    }

    try {
        path = du.getMozUserHomeDir() + path;
        var f = new File(path);

        if (f.exists()) {
        f.open();
        var bundle = f.read();
        f.close();

        if (bundle == "") {
            dump("bundle is empty in getBundle\n");
            return false;
        }

        fontFace = getProperty(bundle,"font.face");
        fontSize = getProperty(bundle,"font.size");
        fontColor = getProperty(bundle,"font.color");
        backgroundColor = getProperty(bundle,"background.color");

        } else {
            dump("properties file could not be opened in getBundle();\n");
            return false;
        }
    } catch(e) {
        dump(e + "error in getBundle\n");
        return false;
    }
}


// save history file
function saveHistory() {
    var propStr = "";
	var du = new DirUtils();
    var unixPath = "/chrome/mozeditproperties/history.properties";
    var winPath = "\\chrome\\mozeditproperties\\history.properties";
    var path = unixPath;

	if (navigator.platform == "Win32") {
	    path = winPath;
    }

	path = du.getMozUserHomeDir() + path;
    var f = new File(path);

	for (x = 0; x < documentHistory.length; x++) {
	    var historyItem = documentHistory[x];
		if (historyItem == "") {
		    continue;
		}
        propStr = propStr + historyItem + "\n";
        if (navigator.platform == "Win32") {
            propStr = propStr.replace(/\//g,"\\");
        }

    }

    if ( f.exists() ) {
       try {
           f.open('w');
	       f.write(propStr);
	       f.close();
	   } catch(e) {
	       dump(e +"writing to file in saveHistory()\n");
           return false;
	   }
	   updateStatus("History saved...");
   } else {
       try {
       f.create();
	   if (f.exists()) {
	       f.open('w');
	       f.write(propStr);
	       updateStatus("File saved...");
	   } else {
	       dump("History could not be saved in saveHistory()\n")
	   }
	   f.close();
       } catch(e) {
           dump(e+"error occured in saveHistory()\n");
           return false;
       }
   }
}


// clear history
function clearHistory() {
	var menuList = document.getElementById('menu-go-list');
    var menuPopup = document.getElementById('menu-go-popup');
	var kids = historyItems;
    for ( x = 0; x <= historyItems+2; x++) {
	    try {
	        var currentChild = document.getElementById( "menu-go-item"+x);
            throwaway_node = menuPopup.removeChild( currentChild );
		} catch (e) {
		    dump("error in modifying nodes in clearHistory()\n");
		}
	}
   historyItems = 0;
   documentHistory = new Array();
   saveHistory();
}


// add file to history menu
function addToHistory() {
    if ( documentHistory.length < historyMax && inputFile.value != "" ) {
	    if ( searchArray(inputFile.value, documentHistory) ) {
            documentHistory.push(inputFile.value);
		    appendGoMenu();
		    saveHistory();
		}
	}
}

// loads in bundle from history
function getHistory() {
    var du = new DirUtils();
    var unixPath = "/chrome/mozeditproperties/history.properties";
    var winPath = "\\chrome\\mozeditproperties\\history.properties";
    var path = unixPath;

    if (navigator.platform == "Win32") {
          path = winPath
    }

    try {
        path = du.getMozUserHomeDir() + path;
        dump("user home dir from getHistory() \n"+ path);
        var f = new File(path);

        if (f.exists()) {
            f.open();
            var bundle = f.read();
            f.close();
            documentHistory = bundle.split("\n");
        } else {
            dump("history file could not be opened in getHistory();\n");
        }
     } catch(e) {
        dump(e+"error in getHistory()");
        return false;
    }
}

// add History menu items
function buildGoMenu() {;
    var menuList = document.getElementById('menu-go-list');
    var menuPopup = document.getElementById('menu-go-popup');
    for (x = 0; x < documentHistory.length; x++) {
        var menuItem = trim(documentHistory[x]);

	    if ( menuItem == "") { continue };

        path = documentHistory[x];
        if (navigator.platform == "Win32") {
            path = path.replace(/\//g,"\\");
        }
        path = encodeURI(path);
		var menuItem = document.createElement('menuitem');

		menuPopup.appendChild(menuItem);
		menuItem.setAttribute('label',documentHistory[x]);
		menuItem.setAttribute('oncommand',"goToURL('" + path + "')");
		menuItem.setAttribute('id',"menu-go-item"+x);
		dump("adding: menu-go-item"+x+"\n");
		historyItems++;
	  }
}

// add History menu items
// todo
function appendGoMenu() {;
    var menuPopup = document.getElementById('menu-go-popup');
    var menuItem = document.createElement('menuitem');
    
     path = inputFile.value;
     if (navigator.platform == "Win32") {
         path = path.replace(/\//g,"\\");
     }
     path = encodeURI(path);


	 //remove oldest file in go menu
	 //dump("historyItems: "+historyItems+"  historyMax: "+historyMax);
	//if ( historyItems >= historyMax ) {
	   // var oldItem = document.getElementById( 'menu-go-item1');
       // throwaway_node = menuPopup.removeChild( oldItem );
	    //dump("removed node: menu-go-item1\n");
    //}
	menuPopup.appendChild(menuItem);
	menuItem.setAttribute('label',inputFile.value);
	menuItem.setAttribute('oncommand',"goToURL('" + path + "')");
	menuItem.setAttribute('id',"menu-go-item" + historyItems);
	dump("adding: menu-go-item"+historyItems+"\n");
	historyItems++;
}


function setWindowColor(fontColor) {
    this.fontColor = fontColor;
    restoreUIPrefs();
}

// changes font size of text area
function setFontSize(fontSize) {
    this.fontSize = fontSize;
    contentCopy = content.value;

    content.value = "";
    restoreUIPrefs();
    content.value = contentCopy;
    contentCopy = null;
}

// changes font size of text area
function setFontFace(face) {
    this.fontFace = face;
    contentCopy = content.value;

    content.value = "";
    restoreUIPrefs();
    content.value = contentCopy;
    contentCopy = null;
}


// center window on screen
function centerWindowOnScreen() {
    var APP_WIDTH = 750;
	var APP_HEIGHT = 650;
    var width = (screen.availWidth - APP_WIDTH) * .5;
    var height = (screen.availHeight - APP_HEIGHT) * .5;

    window.moveTo(width,height);
}

// closes app window
function closeWindow() {
    window.close();
}


// updates status bar at bottom of app window
function updateStatus(updateMsg) {
    status.value = "";
    status.value = "Status: " + updateMsg;
}


// Print content
function doPrint() {
	if ( browser.getAttribute('hidden').search('false') != -1 ) {
	    refreshPreview();
        browser.contentWindow.print();
	} else {
	    renderPreview();
		browser.contentWindow.print();
	}
}


// refresh browser area pane
function renderPreview() {
    var splitter = document.getElementById('splitter-preview');

	if ( browser.getAttribute('hidden').search('false') != -1 ) {
	    splitter.setAttribute('hidden','true');
	    browser.setAttribute('hidden','true');
	} else {
	    splitter.setAttribute('hidden','false');
	    browser.setAttribute('hidden','false');
	}
	refreshPreview();
}


// refresh preview
function refreshPreview() {
    browserFrame = window.frames[0].document;
    if ( console.getAttribute('hidden').search('false') != -1 ) {
	     browserFrame = window.frames[1].document;
	}
    browserFrame.open();
    browserFrame.write(content.value);
    browserFrame.close();
    updateStatus("Browser preview refreshed..");
}

// hide/show left pane
function toggleConsole() {
    splitter = document.getElementById('splitter-console');
	console = document.getElementById('javascript-console');
	if ( console.getAttribute('hidden').search('false') != -1 ) {
	    splitter.setAttribute('hidden','true');
	    console.setAttribute('hidden','true');
	} else {
	
	    splitter.setAttribute('hidden','false');
	    console.setAttribute('hidden','false');
	}
}

// hide/show left pane
function toggleLineNumbers() {
    setLineNumbers();
	if ( lines.getAttribute('hidden').search('false') != -1 ) {
	    lines.setAttribute('hidden','true');
	} else {
	    lines.setAttribute('hidden','false');
	}
}


// get count of lines
function getLineCount() {
    var count = 0;
	var text = content.value;

    for (var x=0; x < text.length; x++) {
	    if (text.charCodeAt(x) == 10) { //10 is newline char
		    count++;
		}
	}
	count++;
	text = null;
    updateStatus("Line count: "+count);
}


// fill in left pane with numbers
function setLineNumbers() {
    var temp = "";
	for (x=1; x < getLineCount() + 2; x++) {
	    temp = temp + x + "\n";
	}
	lines.value = temp;
}



// clear all document text
function clear() {
	contentCopy = content.value;
	content.value="";
	updateStatus("Content cleared; revert available.");
}

function revert() {
    if (contentCopy != null && contentCopy != "") {
	    content.value = contentCopy;
	    contentCopy = null;
        updateStatus("Reverted operation...");
     } else {
	     updateStatus("Revert unavailable...");
     }
}


// open find XUL dialog
function doFindDialog() {
    content.focus();
    window.openDialog("chrome://mozedit/content/findDialog.xul",
	    "find","chrome,dialog,modal",getDialogXY(320));
}


// helper function returns XY screen coordinates
function getDialogXY(width) {
    if (width == null || width == "") {
	    width = 0;
	}
    var obj = new Object();
	obj.y = window.screenY+90;
	obj.x = window.screenX + window.outerWidth * .5 - (width*.5);
	return obj;
}

// open replace XUL dialog
function doReplaceDialog() {
    window.openDialog("chrome://mozedit/content/replaceDialog.xul",
	    "replace","chrome,dialog,modal",getDialogXY(320));
}

// preferences dialog
function doPrefsDialog() {
    var win = window.openDialog("chrome://mozedit/content/prefsDialog.xul",
	    "about","chrome,dialog,modal,",getDialogXY(450));
}

// open replace XUL dialog
function doHelp() {
    window.openDialog("chrome://mozedit/content/helpDialog.xul",
	    "help","chrome,dialog",getDialogXY(500));
}


// about dialog
function doAbout() {
    var win = window.openDialog("chrome://mozedit/content/about.xul",
	    "about","chrome,dialog,modal",getDialogXY(320));
}


function newMozeditWindow()
{
    // Open mozedit window
    window.openDialog("chrome://mozedit/content", "_blank", "chrome,all,dialog=no");
}


function newFirebirdWindow()
{
    // Open mozedit window
    window.openDialog("chrome://browser/content", "_blank", "chrome,all,dialog=no");
}



// goto files section
function goToURL(path) {
    inputFile.value = decodeURI(path);
    openFile();
}

function goChrome() {
    var f = new FileUtils();
    var du = new DirUtils();
    var unixPath = "/chrome/userChrome.css";
    var winPath = "\\chrome\\userChrome.css";
    var unixChrome = "/chrome/mozedit/content/resources/userChrome.css";
    var winChrome = "\\chrome\\mozedit\\content\\resources\\userChrome.css";
	var path = unixPath;
    var chromePath = unixChrome;

    if (navigator.platform == "Win32") {
	    path = winPath;
        chromePath = winChrome;
    }
	try {
         var homePath = du.getMozUserHomeDir();
         path = homePath + path;
         chromePath = homePath + chromePath;
         
	     inputFile.value = path;

	     if (! openFile() ) {
               if ( confirm("Would you like Mozedit to attempt to install your default userChrome.css file? ") )                {
                  f.copy( chromePath, path);
                  openFile();
               }
         }
	} catch (e) {
	     dump("userChrome.css could not be opened in goChrome()\n");
	     return false;
	}
	return true;
}


function goContent() {
    var f = new FileUtils();
    var du = new DirUtils();
    var unixPath = "/chrome/userContent.css";
    var winPath = "\\chrome\\userContent.css";
    var unixChrome = "/chrome/mozedit/content/resources/userContent.css";
    var winChrome = "\\chrome\\mozedit\\content\\resources\\userContent.css";
	var path = unixPath;
    var chromePath = unixChrome;

    if (navigator.platform == "Win32") {
	    path = winPath;
        chromePath = winChrome;
    }
	try {
         var homePath = du.getMozUserHomeDir();
         path = homePath + path;
         chromePath = homePath + chromePath;
	     inputFile.value = path;

	     if (! openFile() ) {
            if ( confirm("Would you like Mozedit to attempt to install your default userContent.css file? ") ) {
                f.copy( chromePath, path);
                openFile();
            }
         }
	} catch (e) {
	    dump("userContent.css could not be opened in goContent()\n"+e);
		return false;
	}
	return true;
}







